"""Incoming Webhooks

This module contains actions for sending messages & approval requests to a Teams channel.

!!! tip "Before you Start"

    Ensure you have enabled incoming webhooks on at least one Teams channel.

    See the [Integration](../integration/) page for more information
"""
import requests
import logging
import datetime
import shortuuid

from ppa import appliance, task, events


from . import exceptions
from .records import ApprovalResponse


logger = logging.getLogger(__name__)


def _check_status_code(response) -> requests.Response:
    status_code = response.status_code

    if status_code == 400:
        try:
            logger.error(response.json())
        except Exception:
            logger.error(response.text)
        raise exceptions.InvalidMessage("The supplied message format is invalid.")

    elif status_code in {403, 404}:
        logger.error(response.text)
        raise exceptions.InvalidWebhook(
            f"Requested to the supplied webhook URL returned status code '{status_code}' with message '{response.text}'."
        )

    return response


def send_message(url: str, message: str) -> None:
    """Send a message to a Teams channel.

!!! question "Teams Channel"

    The message will be send to the channel associated with the supplied webhook URL.

??? info "Input"

    - __url:__ the Teams incoming webhook URL

    - __message:__ the message to send to the channel

??? success "Output"

    Nothing is outputted by this action.

??? example

    ```YAML linenums="1"
    - teams.incoming_webhooks.send_message:
        message: Hello from PPA!
      load:
        url: teams_secrets.webhook_url
    ```

!!! warning "Supplying Sensitive Data"

    You should ___always___ use a PPA Vault integration to provide sensitive data to a plugin action.
    """
    _check_status_code(
        requests.post(
            url,
            json={
                "@context": "https://schema.org/extensions",
                "@type": "MessageCard",
                "sections": [{"text": message}],
                "themeColor": "0072C6",
                "summary": "Message from a PPA task",
            },
        )
    )


def send_approval_request(url: str, message: str) -> ApprovalResponse:
    """Send an approval request to a Teams channel & wait until it approved/rejected.

Once the request is responded to, a message will be posted to Teams containing the following:

- Whether the request was approved or rejected
- The username of the person who responded to the request

Example screenshots of all the Teams messages can be found under the example.

!!! question "Teams Channel"

    The message will be sent to the channel associated with the supplied webhook URL.

??? info "Input"

    - __url:__ the Teams incoming webhook URL

    - __message:__ a message to include in the approval request

??? success "Output"

    A dictionary with the following keys:

    - __approved:__ `true` if the request was approved, `false` if it was not

    - __responder:__ the PPA user who submitted the response

??? example

    ```YAML linenums="1"
    - teams.incoming_webhooks.send_approval_request:
        message: Add user to Remote Desktop Users group.
      load:
        url: teams_secrets.webhook_url
      save: approval_response
    ```

??? note "Approval Request Screenshot"

    <img src="../../../images/screenshots/teams/approval_request.png" alt="approval-request" width="100%"/>

??? note "Approved Message Screenshot"

    <img src="../../../images/screenshots/teams/request_approved.png" alt="requested-approved" width="100%"/>

??? note "Rejected Message Screenshot"

    <img src="../../../images/screenshots/teams/request_rejected.png" alt="request-rejected" width="100%"/>

!!! warning "Supplying Sensitive Data"

    You should ___always___ use a PPA Vault integration to provide sensitive data to a plugin action.
    """
    wait_hook_url, secret = events.create_wait_hook()
    approve_link = f"{wait_hook_url}?approve=true"
    reject_link = f"{wait_hook_url}?approve=false"
    task_link = f"https://{appliance.address()}/ui/activity/attach/?uuid={wait_hook_url.split('/')[-2]}"
    task_name = task.manifest()["labels"]["ppa.name"]
    request_uuid = shortuuid.uuid()

    try:
        started_by = task.user().split("\\")[1]
    except IndexError:
        started_by = task.user()

    request_payload = {
        "@context": "https://schema.org/extensions",
        "@type": "MessageCard",
        "summary": "New PPA Approval Request",
        "potentialAction": [
            {"@type": "OpenUri", "name": "Approve", "targets": [{"uri": f"{approve_link}", "os": "default"}]},
            {"@type": "OpenUri", "name": "Reject", "targets": [{"uri": f"{reject_link}", "os": "default"}]},
        ],
        "sections": [
            {
                "text": "**Request Info:**",
                "facts": [
                    {"name": "Task:", "value": f"{task_name}"},
                    {"name": "Operator:", "value": f"{started_by}"},
                    {"name": "Requested At", "value": datetime.datetime.utcnow().strftime("%H:%M:%S %d/%m/%Y")},
                    {"name": "Unique Request ID", "value": f"{request_uuid}"},
                ],
            },
            {"text": f"Click [here]({task_link}) to view the task."},
            {
                "text": f"""**Request Message:**


{message}"""
            },
            {"text": "**Please Choose an Option:**"},
        ],
        "themeColor": "0072C6",
        "title": "&#x1F514; New PPA Approval Request &#x1F514;",
    }

    logger.debug(request_payload)

    _check_status_code(requests.post(url, json=request_payload))

    approval_response = events.wait("Waiting for Teams approval response...", secret=secret)

    approved = approval_response["values"]["approve"][0].lower() == "true"
    try:
        responder = approval_response["username"].split("\\")[1]
    except IndexError:
        responder = approval_response["username"]

    if approved:
        _check_status_code(
            requests.post(
                url,
                json={
                    "@context": "https://schema.org/extensions",
                    "@type": "MessageCard",
                    "summary": "PPA Approval Request Was Approved",
                    "sections": [
                        {
                            "text": "**Request Info:**",
                            "facts": [
                                {"name": "Approved By:", "value": f"{responder}"},
                                {"name": "Task:", "value": f"{task_name}"},
                                {"name": "Unique Request ID", "value": f"{request_uuid}"},
                            ],
                        }
                    ],
                    "themeColor": "0072C6",
                    "title": "&#x1F3C6; Request Approved &#x1F3C6;",
                },
            )
        )
    else:
        _check_status_code(
            requests.post(
                url,
                json={
                    "@context": "https://schema.org/extensions",
                    "@type": "MessageCard",
                    "summary": "PPA Approval Request Was Rejected",
                    "sections": [
                        {
                            "text": "**Request Info:**",
                            "facts": [
                                {"name": "Rejected By:", "value": f"{responder}"},
                                {"name": "Task:", "value": f"{task_name}"},
                                {"name": "Unique Request ID", "value": f"{request_uuid}"},
                            ],
                        }
                    ],
                    "themeColor": "0072C6",
                    "title": "&#x1F6AB; Request Rejected &#x1F6AB;",
                },
            )
        )

    return ApprovalResponse(approved=approved, responder=responder)
